from setuptools import setup

setup(
    name='aadb',
    version='1.0.0',
    packages=['aadb'],
    url='',
    license='MIT',
    author='fuxingfu',
    author_email='fxfpro@163.com',
    description='async with android debug bridge'
)
